@@ -21,6 +21,31 @@
 /**
  * SECTION:gesaudiosource
  * @short_description: Base Class for audio sources
+ *
+ * <refsect1 id="GESVideoSource.children_properties" role="properties">
+ * <title role="children_properties.title">Children Properties</title>
+ * <para>You can use the following children properties through the
+ * #ges_track_element_set_child_property and alike set of methods:</para>
+ * <informaltable frame="none">
+ * <tgroup cols="3">
+ * <colspec colname="properties_type" colwidth="150px"/>
+ * <colspec colname="properties_name" colwidth="200px"/>
+ * <colspec colname="properties_flags" colwidth="400px"/>
+ * <tbody>
+ * <row>
+ *  <entry role="property_type"><link linkend="gdouble"><type>double</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESAudioSource--volume">volume</link></entry>
+ *  <entry>volume factor, 1.0=100%.</entry>
+ * </row>
+ * <row>
+ *  <entry role="property_type"><link linkend="gboolean"><type>gboolean</type></link></entry>
+ *  <entry role="property_name"><link linkend="GESAudioSource--mute">mute</link></entry>
+ *  <entry>mute channel.</entry>
+ * </row>
+ * </tbody>
+ * </tgroup>
+ * </informaltable>
+ * </refsect1>
  */
 
 #include "ges-internal.h"
@@ -101,7 +126,7 @@ ges_audio_source_class_init (GESAudioSourceClass * klass)
 
   g_type_class_add_private (klass, sizeof (GESAudioSourcePrivate));
 
-  track_class->gnlobject_factorytype = "gnlsource";
+  track_class->nleobject_factorytype = "nlesource";
   track_class->create_element = ges_audio_source_create_element;
   audio_source_class->create_source = NULL;
 }
