@@ -21,6 +21,10 @@
 #include "ges.h"
 #include "ges-internal.h"
 
+GST_DEBUG_CATEGORY_STATIC (base_xml_formatter);
+#undef GST_CAT_DEFAULT
+#define GST_CAT_DEFAULT base_xml_formatter
+
 #define parent_class ges_base_xml_formatter_parent_class
 G_DEFINE_ABSTRACT_TYPE (GESBaseXmlFormatter, ges_base_xml_formatter,
     GES_TYPE_FORMATTER);
@@ -167,10 +171,17 @@ create_parser_context (GESBaseXmlFormatter * self, const gchar * uri,
 
   GError *err = NULL;
 
-  if ((file = g_file_new_for_uri (uri)) == NULL)
-    goto wrong_uri;
+  GST_DEBUG_OBJECT (self, "loading xml from %s", uri);
+
+  file = g_file_new_for_uri (uri);
 
   /* TODO Handle GCancellable */
+  if (!g_file_query_exists (file, NULL)) {
+    err = g_error_new (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
+        "Invalid URI: \"%s\"", uri);
+    goto failed;
+  }
+
   if (!g_file_load_contents (file, NULL, &xmlcontent, &xmlsize, NULL, &err))
     goto failed;
 
@@ -187,21 +198,15 @@ create_parser_context (GESBaseXmlFormatter * self, const gchar * uri,
   if (!g_markup_parse_context_end_parse (parsecontext, &err))
     goto failed;
 
-done:
-  if (xmlcontent)
-    g_free (xmlcontent);
 
-  if (file)
-    gst_object_unref (file);
+done:
+  g_free (xmlcontent);
+  g_object_unref (file);
 
   return parsecontext;
 
-wrong_uri:
-  GST_WARNING ("%s wrong uri", uri);
-
-  goto done;
-
 failed:
+  GST_WARNING ("failed to load contents from \"%s\"", uri);
   g_propagate_error (error, err);
 
   if (parsecontext) {
@@ -407,6 +412,9 @@ ges_base_xml_formatter_class_init (GESBaseXmlFormatterClass * self_class)
   formatter_klass->save_to_uri = _save_to_uri;
 
   self_class->save = NULL;
+
+  GST_DEBUG_CATEGORY_INIT (base_xml_formatter, "base-xml-formatter",
+      GST_DEBUG_FG_BLUE | GST_DEBUG_BOLD, "Base XML Formatter");
 }
 
 /***********************************************
@@ -478,7 +486,7 @@ static gboolean
 _loading_done_cb (GESFormatter * self)
 {
   _loading_done (self);
-  g_object_unref (self);
+  gst_object_unref (self);
 
   return FALSE;
 }
@@ -1007,9 +1015,6 @@ ges_base_xml_formatter_add_track (GESBaseXmlFormatter * self,
   GESBaseXmlFormatterPrivate *priv = _GET_PRIV (self);
 
   if (priv->check_only) {
-    if (caps)
-      gst_caps_unref (caps);
-
     return;
   }
 
@@ -1018,18 +1023,20 @@ ges_base_xml_formatter_add_track (GESBaseXmlFormatter * self,
 
   if (properties) {
     gchar *restriction;
-    GstCaps *caps;
+    GstCaps *restriction_caps;
 
     gst_structure_get (properties, "restriction-caps", G_TYPE_STRING,
         &restriction, NULL);
     gst_structure_remove_fields (properties, "restriction-caps", "caps",
         "message-forward", NULL);
     if (g_strcmp0 (restriction, "NULL")) {
-      caps = gst_caps_from_string (restriction);
-      ges_track_set_restriction_caps (track, caps);
+      restriction_caps = gst_caps_from_string (restriction);
+      ges_track_set_restriction_caps (track, restriction_caps);
+      gst_caps_unref (restriction_caps);
     }
     gst_structure_foreach (properties,
         (GstStructureForeachFunc) set_property_foreach, track);
+    g_free (restriction);
   }
 
   g_hash_table_insert (priv->tracks, g_strdup (id), gst_object_ref (track));
