@@ -41,6 +41,7 @@
 G_DEFINE_TYPE (GESGroup, ges_group, GES_TYPE_CONTAINER);
 
 #define GES_CHILDREN_INIBIT_SIGNAL_EMISSION (GES_CHILDREN_LAST + 1)
+#define GES_GROUP_SIGNALS_IDS_DATA_KEY_FORMAT "ges-group-signals-ids-%p"
 
 struct _GESGroupPrivate
 {
@@ -53,13 +54,26 @@ struct _GESGroupPrivate
   gboolean setting_value;
 };
 
+typedef struct
+{
+  GESLayer *layer;
+  gulong child_clip_changed_layer_sid;
+  gulong child_priority_changed_sid;
+  gulong child_group_priority_changed_sid;
+} ChildSignalIds;
+
 enum
 {
   PROP_0,
+  PROP_START,
+  PROP_INPOINT,
+  PROP_DURATION,
+  PROP_MAX_DURATION,
+  PROP_PRIORITY,
   PROP_LAST
 };
 
-/* static GParamSpec *properties[PROP_LAST]; */
+static GParamSpec *properties[PROP_LAST] = { NULL, };
 
 /****************************************************
  *              Our listening of children           *
@@ -107,13 +121,59 @@ _update_our_values (GESGroup * group)
       max_layer_prio - min_layer_prio + 1);
 }
 
+static void
+_child_priority_changed_cb (GESLayer * layer,
+    GParamSpec * arg G_GNUC_UNUSED, GESTimelineElement * clip)
+{
+  GESContainer *container = GES_CONTAINER (GES_TIMELINE_ELEMENT_PARENT (clip));
+
+  gint layer_prio = ges_layer_get_priority (layer);
+  gint offset = _ges_container_get_priority_offset (container, clip);
+
+  if (container->children_control_mode != GES_CHILDREN_UPDATE) {
+    GST_DEBUG_OBJECT (container, "Ignoring updated");
+    return;
+  }
+
+  if (layer_prio + offset == _PRIORITY (container))
+    return;
+
+  container->initiated_move = clip;
+  _set_priority0 (GES_TIMELINE_ELEMENT (container), layer_prio + offset);
+  container->initiated_move = NULL;
+}
+
 static void
 _child_clip_changed_layer_cb (GESTimelineElement * clip,
     GParamSpec * arg G_GNUC_UNUSED, GESGroup * group)
 {
+  ChildSignalIds *sigids;
+  gchar *signals_ids_key;
+  GESLayer *old_layer, *new_layer;
   gint offset, layer_prio = ges_clip_get_layer_priority (GES_CLIP (clip));
   GESContainer *container = GES_CONTAINER (group);
 
+  offset = _ges_container_get_priority_offset (container, clip);
+  signals_ids_key =
+      g_strdup_printf (GES_GROUP_SIGNALS_IDS_DATA_KEY_FORMAT, clip);
+  sigids = g_object_get_data (G_OBJECT (group), signals_ids_key);
+  g_free (signals_ids_key);
+  old_layer = sigids->layer;
+
+  new_layer = ges_clip_get_layer (GES_CLIP (clip));
+
+  if (sigids->child_priority_changed_sid) {
+    g_signal_handler_disconnect (old_layer, sigids->child_priority_changed_sid);
+    sigids->child_priority_changed_sid = 0;
+  }
+
+  if (new_layer) {
+    sigids->child_priority_changed_sid =
+        g_signal_connect (new_layer, "notify::priority",
+        (GCallback) _child_priority_changed_cb, clip);
+  }
+  sigids->layer = new_layer;
+
   if (container->children_control_mode != GES_CHILDREN_UPDATE) {
     if (container->children_control_mode == GES_CHILDREN_INIBIT_SIGNAL_EMISSION) {
       container->children_control_mode = GES_CHILDREN_UPDATE;
@@ -122,15 +182,10 @@ _child_clip_changed_layer_cb (GESTimelineElement * clip,
     return;
   }
 
-  offset = _ges_container_get_priority_offset (container, clip);
-
-  if (layer_prio + offset < 0 ||
-      (GES_TIMELINE_ELEMENT_TIMELINE (group) &&
-          layer_prio + offset + GES_CONTAINER_HEIGHT (group) - 1 >
-          g_list_length (GES_TIMELINE_ELEMENT_TIMELINE (group)->layers))) {
-    GESLayer *old_layer =
-        g_list_nth_data (GES_TIMELINE_ELEMENT_TIMELINE (group)->layers,
-        _PRIORITY (group) - offset);
+  if (new_layer && (layer_prio + offset < 0 ||
+          (GES_TIMELINE_ELEMENT_TIMELINE (group) &&
+              layer_prio + offset + GES_CONTAINER_HEIGHT (group) - 1 >
+              g_list_length (GES_TIMELINE_ELEMENT_TIMELINE (group)->layers)))) {
 
     GST_INFO_OBJECT (container, "Trying to move to a layer outside of"
         "the timeline layers, moving back to old layer (prio %i)",
@@ -378,6 +433,8 @@ static void
 _child_added (GESContainer * group, GESTimelineElement * child)
 {
   GList *children, *tmp;
+  gchar *signals_ids_key;
+  ChildSignalIds *signals_ids;
 
   GESGroupPrivate *priv = GES_GROUP (group)->priv;
   GstClockTime last_child_end = 0, first_child_start = G_MAXUINT64;
@@ -410,33 +467,73 @@ _child_added (GESContainer * group, GESTimelineElement * child)
   group->children_control_mode = GES_CHILDREN_UPDATE;
   _update_our_values (GES_GROUP (group));
 
+  signals_ids_key =
+      g_strdup_printf (GES_GROUP_SIGNALS_IDS_DATA_KEY_FORMAT, child);
+  signals_ids = g_malloc0 (sizeof (ChildSignalIds));
+  g_object_set_data_full (G_OBJECT (group), signals_ids_key,
+      signals_ids, g_free);
+  g_free (signals_ids_key);
   if (GES_IS_CLIP (child)) {
-    g_signal_connect (child, "notify::layer",
+    GESLayer *layer = ges_clip_get_layer (GES_CLIP (child));
+
+    signals_ids->child_clip_changed_layer_sid =
+        g_signal_connect (child, "notify::layer",
         (GCallback) _child_clip_changed_layer_cb, group);
+
+    if (layer) {
+      signals_ids->child_priority_changed_sid = g_signal_connect (layer,
+          "notify::priority", (GCallback) _child_priority_changed_cb, child);
+    }
+    signals_ids->layer = layer;
+
   } else if (GES_IS_GROUP (child), group) {
-    g_signal_connect (child, "notify::priority",
+    signals_ids->child_group_priority_changed_sid =
+        g_signal_connect (child, "notify::priority",
         (GCallback) _child_group_priority_changed, group);
   }
 }
 
+static void
+_disconnect_signals (GESGroup * group, GESTimelineElement * child,
+    ChildSignalIds * sigids)
+{
+  if (sigids->child_group_priority_changed_sid) {
+    g_signal_handler_disconnect (child,
+        sigids->child_group_priority_changed_sid);
+    sigids->child_group_priority_changed_sid = 0;
+  }
+
+  if (sigids->child_clip_changed_layer_sid) {
+    g_signal_handler_disconnect (child, sigids->child_clip_changed_layer_sid);
+    sigids->child_clip_changed_layer_sid = 0;
+  }
+
+  if (sigids->child_priority_changed_sid) {
+    g_signal_handler_disconnect (sigids->layer,
+        sigids->child_priority_changed_sid);
+    sigids->child_priority_changed_sid = 0;
+  }
+}
+
+
 static void
 _child_removed (GESContainer * group, GESTimelineElement * child)
 {
   GList *children;
   GstClockTime first_child_start;
+  gchar *signals_ids_key;
+  ChildSignalIds *sigids;
   GESGroupPrivate *priv = GES_GROUP (group)->priv;
 
   _ges_container_sort_children (group);
 
   children = GES_CONTAINER_CHILDREN (group);
 
-  if (GES_IS_CLIP (child))
-    g_signal_handlers_disconnect_by_func (child, _child_clip_changed_layer_cb,
-        group);
-  else if (GES_IS_GROUP (child), group)
-    g_signal_handlers_disconnect_by_func (child, _child_group_priority_changed,
-        group);
-
+  signals_ids_key =
+      g_strdup_printf (GES_GROUP_SIGNALS_IDS_DATA_KEY_FORMAT, child);
+  sigids = g_object_get_data (G_OBJECT (group), signals_ids_key);
+  _disconnect_signals (GES_GROUP (group), child, sigids);
+  g_free (signals_ids_key);
   if (children == NULL) {
     GST_FIXME_OBJECT (group, "Auto destroy myself?");
     timeline_remove_group (GES_TIMELINE_ELEMENT_TIMELINE (group),
@@ -501,6 +598,23 @@ _group (GList * containers)
   return ret;
 }
 
+static gboolean
+_paste (GESTimelineElement * element, GESTimelineElement * ref,
+    GstClockTime paste_position)
+{
+  if (GES_TIMELINE_ELEMENT_CLASS (parent_class)->paste (element,
+          ref, paste_position)) {
+
+    if (GES_CONTAINER_CHILDREN (element))
+      timeline_add_group (GES_TIMELINE_ELEMENT_TIMELINE (GES_CONTAINER_CHILDREN
+              (element)->data), GES_GROUP (element));
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 
 /****************************************************
  *                                                  *
@@ -511,9 +625,26 @@ static void
 ges_group_get_property (GObject * object, guint property_id,
     GValue * value, GParamSpec * pspec)
 {
+  GESTimelineElement *self = GES_TIMELINE_ELEMENT (object);
+
   switch (property_id) {
+    case PROP_START:
+      g_value_set_uint64 (value, self->start);
+      break;
+    case PROP_INPOINT:
+      g_value_set_uint64 (value, self->inpoint);
+      break;
+    case PROP_DURATION:
+      g_value_set_uint64 (value, self->duration);
+      break;
+    case PROP_MAX_DURATION:
+      g_value_set_uint64 (value, self->maxduration);
+      break;
+    case PROP_PRIORITY:
+      g_value_set_uint (value, self->priority);
+      break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
   }
 }
 
@@ -521,9 +652,26 @@ static void
 ges_group_set_property (GObject * object, guint property_id,
     const GValue * value, GParamSpec * pspec)
 {
+  GESTimelineElement *self = GES_TIMELINE_ELEMENT (object);
+
   switch (property_id) {
+    case PROP_START:
+      ges_timeline_element_set_start (self, g_value_get_uint64 (value));
+      break;
+    case PROP_INPOINT:
+      ges_timeline_element_set_inpoint (self, g_value_get_uint64 (value));
+      break;
+    case PROP_DURATION:
+      ges_timeline_element_set_duration (self, g_value_get_uint64 (value));
+      break;
+    case PROP_PRIORITY:
+      ges_timeline_element_set_priority (self, g_value_get_uint (value));
+      break;
+    case PROP_MAX_DURATION:
+      ges_timeline_element_set_max_duration (self, g_value_get_uint64 (value));
+      break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
   }
 }
 
@@ -544,7 +692,63 @@ ges_group_class_init (GESGroupClass * klass)
   element_class->set_inpoint = _set_inpoint;
   element_class->set_start = _set_start;
   element_class->set_priority = _set_priority;
-  /* TODO implement the deep_copy Virtual method */
+  element_class->paste = _paste;
+
+  /* We override start, inpoint, duration and max-duration from GESTimelineElement
+   * in order to makes sure those fields are not serialized.
+   */
+  /**
+   * GESGroup:start:
+   *
+   * The position of the object in its container (in nanoseconds).
+   */
+  properties[PROP_START] = g_param_spec_uint64 ("start", "Start",
+      "The position in the container", 0, G_MAXUINT64, 0,
+      G_PARAM_READWRITE | GES_PARAM_NO_SERIALIZATION);
+
+  /**
+   * GESGroup:in-point:
+   *
+   * The in-point at which this #GESGroup will start outputting data
+   * from its contents (in nanoseconds).
+   *
+   * Ex : an in-point of 5 seconds means that the first outputted buffer will
+   * be the one located 5 seconds in the controlled resource.
+   */
+  properties[PROP_INPOINT] =
+      g_param_spec_uint64 ("in-point", "In-point", "The in-point", 0,
+      G_MAXUINT64, 0, G_PARAM_READWRITE | GES_PARAM_NO_SERIALIZATION);
+
+  /**
+   * GESGroup:duration:
+   *
+   * The duration (in nanoseconds) which will be used in the container
+   */
+  properties[PROP_DURATION] =
+      g_param_spec_uint64 ("duration", "Duration", "The duration to use", 0,
+      G_MAXUINT64, GST_CLOCK_TIME_NONE,
+      G_PARAM_READWRITE | GES_PARAM_NO_SERIALIZATION);
+
+  /**
+   * GESGroup:max-duration:
+   *
+   * The maximum duration (in nanoseconds) of the #GESGroup.
+   */
+  properties[PROP_MAX_DURATION] =
+      g_param_spec_uint64 ("max-duration", "Maximum duration",
+      "The maximum duration of the object", 0, G_MAXUINT64, GST_CLOCK_TIME_NONE,
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | GES_PARAM_NO_SERIALIZATION);
+
+  /**
+   * GESTGroup:priority:
+   *
+   * The priority of the object.
+   */
+  properties[PROP_PRIORITY] = g_param_spec_uint ("priority", "Priority",
+      "The priority of the object", 0, G_MAXUINT, 0,
+      G_PARAM_READWRITE | GES_PARAM_NO_SERIALIZATION);
+
+  g_object_class_install_properties (object_class, PROP_LAST, properties);
 
   container_class->add_child = _add_child;
   container_class->child_added = _child_added;
